package com.nova.flutter_ads;

import android.app.Activity;
import android.os.Handler;
import android.os.Looper;

import com.ark.adkit.basics.models.ArkVideoCallbacks;
import com.ark.adkit.polymers.polymer.ADTool;
import com.ark.dict.ConfigMapLoader;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BasicMessageChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.StringCodec;

/**
 * FlutterAdsPlugin
 */
public class FlutterAdsPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

    private BasicMessageChannel rewardRunTimeSender;
    private BasicMessageChannel insertRunTimeSender;
    private FlutterPluginBinding pluginBinding;
    private Activity activity;

    public static boolean needPermission = true;

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        switch (call.method) {
            case "initData":
                ConfigMapLoader.getInstance().getData();
                break;
            case "initRewardVideo":
                initRewardAd(call, result);
                break;
            case "initInsertVideo":
                initInsertAd();
                break;
            case "initRewardVideoDirect":
                initRewardAdDirect(call, result);
                break;
            case "initInsertVideoDirect":
                initInsertAdDirect();
                break;
            case "preInitVideo":
                ADTool.getADTool().getManager().getVideoWrapper().preInitReward(activity);
                break;
            case "setConfig":
                Object permission = call.argument("needPermission");
                if (permission != null) {
                    needPermission = (boolean) permission;
                }
                ADTool.Builder builder = new ADTool.Builder();
                Object bannerWidth = call.argument("bannerWidth");
                Object nativeWidth = call.argument("nativeWidth");
                if (bannerWidth != null) {
                    builder.setTtadBannerWidth((Integer) bannerWidth);
                }
                if (nativeWidth != null) {
                    builder.setTtadNativeWidth((Integer) nativeWidth);
                }
                ADTool.initialize(builder.build());
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    @Override
    public void onAttachedToEngine(FlutterPluginBinding flutterPluginBinding) {
        pluginBinding = flutterPluginBinding;
        final MethodChannel channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "flutter_ads");
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {
    }

    @Override
    public void onAttachedToActivity(ActivityPluginBinding binding) {
        activity = binding.getActivity();
        setupViews();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {
    }

    private void setupViews() {
        //注册开屏广告UI插件
        SplashViewFactory factory = new SplashViewFactory(activity, pluginBinding.getBinaryMessenger());
        pluginBinding.getPlatformViewRegistry().registerViewFactory("plugins.nova.ads/splashview", factory);

        //注册本地广告
        NativeViewFactory nativeViewFactory = new NativeViewFactory(activity, pluginBinding.getBinaryMessenger());
        pluginBinding.getPlatformViewRegistry().registerViewFactory("plugins.nova.ads/nativeview", nativeViewFactory);

        //注册banner广告
        BannerViewFactory bannerViewFactory = new BannerViewFactory(activity, pluginBinding.getBinaryMessenger());
        pluginBinding.getPlatformViewRegistry().registerViewFactory("plugins.nova.ads/bannerview", bannerViewFactory);
    }


    Handler handler;

    private void initInsertAd() {
        if (activity == null) {
            return;
        }
        this.handler = new Handler(Looper.getMainLooper());
        insertRunTimeSender = new BasicMessageChannel<>(pluginBinding.getBinaryMessenger(), "plugins.nova.ads.event/insert", StringCodec.INSTANCE);
        ADTool.getADTool().getManager().getVideoWrapper().loadFull(activity, new ArkVideoCallbacks() {
            @Override
            public void onAdShow() {

            }

            @Override
            public void onNoAd() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onAdClose() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onVideoComplete() {

            }

            @Override
            public void onSkippedVideo() {

            }

            @Override
            public void onLoadAdsError(String msg) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onRewardVerify(boolean b, int i, String s) {

            }
        });
    }


    private void initInsertAdDirect() {
        if (activity == null) {
            return;
        }
        this.handler = new Handler(Looper.getMainLooper());
        insertRunTimeSender = new BasicMessageChannel<>(pluginBinding.getBinaryMessenger(), "plugins.nova.ads.event/insert", StringCodec.INSTANCE);
        ADTool.getADTool().getManager().getVideoWrapper().loadFullDirect(activity, new ArkVideoCallbacks() {
            @Override
            public void onAdShow() {

            }

            @Override
            public void onNoAd() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onAdClose() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onVideoComplete() {

            }

            @Override
            public void onSkippedVideo() {

            }

            @Override
            public void onLoadAdsError(String msg) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        insertRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onRewardVerify(boolean b, int i, String s) {

            }
        });
    }


    private void initRewardAd(final MethodCall call, final Result result) {
        if (activity == null) {
            return;
        }
        this.handler = new Handler(Looper.getMainLooper());

        rewardRunTimeSender = new BasicMessageChannel<>(pluginBinding.getBinaryMessenger(), "plugins.nova.ads.event/reward", StringCodec.INSTANCE);

        ADTool.getADTool().getManager().getVideoWrapper().loadReward(activity, new ArkVideoCallbacks() {
            @Override
            public void onAdShow() {

            }

            @Override
            public void onNoAd() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onAdClose() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onVideoComplete() {

            }

            @Override
            public void onSkippedVideo() {

            }

            @Override
            public void onLoadAdsError(String msg) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onRewardVerify(boolean b, int i, String s) {

            }
        });
    }


    private void initRewardAdDirect(final MethodCall call, final Result result) {
        if (activity == null) {
            return;
        }
        this.handler = new Handler(Looper.getMainLooper());

        rewardRunTimeSender = new BasicMessageChannel<>(pluginBinding.getBinaryMessenger(), "plugins.nova.ads.event/reward", StringCodec.INSTANCE);

        ADTool.getADTool().getManager().getVideoWrapper().loadRewardDirect(activity, new ArkVideoCallbacks() {
            @Override
            public void onAdShow() {

            }

            @Override
            public void onNoAd() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onAdClose() {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onVideoComplete() {

            }

            @Override
            public void onSkippedVideo() {

            }

            @Override
            public void onLoadAdsError(String msg) {
                handler.post(new Runnable() {
                    @Override
                    public void run() {
                        rewardRunTimeSender.send("finish");
                    }
                });
            }

            @Override
            public void onRewardVerify(boolean b, int i, String s) {

            }
        });
    }
}
