import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

typedef RewardResult = void Function(bool result);
typedef InsertResult = void Function(bool result);
typedef CpmPopResult = void Function(bool result);

class FlutterAds {
  static const MethodChannel _channel = const MethodChannel('flutter_ads');

  static const BasicMessageChannel<dynamic> rewardMessageChannel =
      const BasicMessageChannel('plugins.nova.ads.event/reward', StringCodec());

  static const BasicMessageChannel<dynamic> insertMessageChannel =
      const BasicMessageChannel('plugins.nova.ads.event/insert', StringCodec());

  static const BasicMessageChannel<dynamic> cpmPopMessageChannel =
      const BasicMessageChannel('plugins.nova.ads.event/cpm', StringCodec());

  static var maxBannerHeight = 130.0;

  static var maxNativeHeight = 330.0;

  static var mbannerMaskColor = Colors.white;
  static var mnativeMaskColor = Colors.white;

  static void initData() {
    if (Platform.isAndroid) {
      _channel.invokeMethod('initData');
    }
  }

  static void setConfig(
      {bool isNeedPermission = true,
      int bannerWidth = 360,
      int nativeWidth = 360,
      Color bannerMaskColor = Colors.white,
      Color nativeMaskColor = Colors.white,
      double bannerHeight,
      double nativeHeight}) async {
    var map = {
      "needPermission": isNeedPermission,
      "bannerWidth": bannerWidth,
      "nativeWidth": nativeWidth,
    };

    if (bannerHeight != null) {
      maxBannerHeight = bannerHeight;
    }
    if (nativeHeight != null) {
      maxNativeHeight = nativeHeight;
    }
    if (bannerMaskColor != null) {
      mbannerMaskColor = bannerMaskColor;
    }
    if (nativeMaskColor != null) {
      mnativeMaskColor = nativeMaskColor;
    }

    _channel.invokeMethod('setConfig', map);
  }

  static void preInitVideo() {
    _channel.invokeMethod('preInitVideo');
  }

  static void initRewardVideo({RewardResult callback}) {
    _channel.invokeMethod('initRewardVideo');
    rewardMessageChannel.setMessageHandler((value) {
      if (value == "finish") {
        callback(true);
      } else {
        callback(false);
      }
    });
  }

  static void initInsertVideo({InsertResult callback}) async {
    _channel.invokeMethod('initInsertVideo');
    insertMessageChannel.setMessageHandler((value) {
      if (value == "finish") {
        callback(true);
      } else {
        callback(false);
      }
    });
  }

  static void initRewardVideoDirect({RewardResult callback}) {
    _channel.invokeMethod('initRewardVideoDirect');
    rewardMessageChannel.setMessageHandler((value) {
      if (value == "finish") {
        callback(true);
      } else {
        callback(false);
      }
    });
  }

  static void initInsertVideoDirect({InsertResult callback}) async {
    _channel.invokeMethod('initInsertVideoDirect');
    insertMessageChannel.setMessageHandler((value) {
      if (value == "finish") {
        callback(true);
      } else {
        callback(false);
      }
    });
  }

  //CPM 弹窗广告
  static void initCpmPopAd({CpmPopResult callback}) async {
    _channel.invokeMethod('initCpmPopAd');
    cpmPopMessageChannel.setMessageHandler((value) {
      if (value == "finish") {
        callback(true);
      } else {
        callback(false);
      }
    });
  }
}
